﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Cat : MonoBehaviour
{
	public int m_catiitudePoints = 0;

	public float m_maxSpeed = 1.0f;
	public float m_movementSpeed = 365.0f;
	public float m_jumpForce = 300.0f;

	private bool m_facingRight = true;

	public bool m_grounded;
	private bool m_jumping;

	private World m_world;
	private Animator m_animator;

	private Transform m_groundCheck;

	private GameObject m_actionBar;

	public BoxCollider2D m_collider;

	public Dictionary<string,Action> m_actions = new Dictionary<string, Action>();

	void Awake()
	{
		m_world = GameObject.FindGameObjectWithTag("_World").GetComponent<World>();
		m_animator = this.GetComponent<Animator>();

		m_collider = this.GetComponent<BoxCollider2D>();

		m_groundCheck = this.transform.Find("Grounded").transform;

		m_actionBar = GameObject.Find("ActionBar");

		m_actions.Add ("Barf",new Barf("Barf",this.transform,5.0f));
		m_actions.Add ("Poop",new Poop("Poop",this.transform,5.0f));

		m_actions.Add ("Attack",new Action("Attack",1.0f));

		m_actions.Add ("Eat",new Eat("Eat",1.0f));
		//m_actions.Add ("Pickup",new Pickup("Pickup",this.transform,1.0f));

		m_actions.Add ("Bite",new Action("Bite",1.0f));
		m_actions.Add ("Hiss",new Action("Hiss",1.0f));
		m_actions.Add ("Stare",new Action("Stare",1.0f));
		m_actions.Add ("KnockOver",new Action("KnockOver",1.0f));
		m_actions.Add ("Sit",new Action("Sit",1.0f));

		m_actions.Add ("Ignore",new Action("Ignore",1.0f));
		m_actions.Add ("Sleep",new Action("Sleep",1.0f));
		m_actions.Add ("Lick",new Lick("Lick",1.0f));

		foreach(KeyValuePair<string,Action> l_action in m_actions)
		{
			l_action.Value.m_world = m_world;
			l_action.Value.m_cat = this;
		}
	}

	void Update ()
	{
		m_grounded = Physics2D.Linecast(this.transform.position,m_groundCheck.position,1 << LayerMask.NameToLayer("Ground"));

		if(Input.GetAxis("Jump") > 0.0f && m_grounded)
		{
			m_jumping = true;

			DisableObstaclesAbove();
			Invoke ("EnableObstacles",0.6f);
		}

		if(Input.GetAxis("Jump") < 0.0f && m_grounded)
		{
			DisableObstaclesBelow();
			Invoke ("EnableObstacles",0.6f);
		}

		foreach(KeyValuePair<string,Action> l_action in m_actions)
		{
			l_action.Value.Check();
		}

		m_animator.SetBool("Grounded",m_grounded);

		UpdateActionBar();
	}

	void FixedUpdate()
	{
		float h = Input.GetAxis("Move");

		m_animator.SetFloat("Speed",Mathf.Abs(h));

		if(h * this.rigidbody2D.velocity.x < m_maxSpeed)
		{
			this.rigidbody2D.AddForce(Vector2.right * h * m_movementSpeed);
		}

		if(Mathf.Abs(this.rigidbody2D.velocity.x) > m_maxSpeed)
		{
			this.rigidbody2D.velocity = new Vector2(Mathf.Sign(this.rigidbody2D.velocity.x) * m_maxSpeed, this.rigidbody2D.velocity.y);
		}

		if(h > 0 && !m_facingRight)
		{
			Flip();
		}
		else if(h < 0 && m_facingRight)
		{
			Flip();
		}

		foreach(KeyValuePair<string,Action> l_action in m_actions)
		{
			l_action.Value.Run(m_animator);
		}

		if(m_jumping)
		{
			m_animator.SetTrigger("Jump");

			this.rigidbody2D.AddForce(Vector2.up * m_jumpForce);

			m_jumping = false;
		}
	}

	public Action GetAction(string name)
	{
		if(m_actions.ContainsKey(name))
		{
			return m_actions[name];
		}

		return null;
	}
	
	void Flip ()
	{
		m_facingRight = !m_facingRight;

		Vector3 l_scale = this.transform.localScale;
		l_scale.x *= -1;
		this.transform.localScale = l_scale;
	}

	void UpdateActionBar()
	{
		foreach(KeyValuePair<string,Action> l_action in m_actions)
		{
			Transform go = m_actionBar.transform.Find ("Action_" + l_action.Key);

			if(go != null)
			{

				Action act = l_action.Value;

				if(act.m_hasTimer)
				{
					Transform disabledTexture = go.Find("DisabledTexture");

					if(!act.m_enabled)
					{
						disabledTexture.GetComponent<SpriteRenderer>().enabled = true;
						disabledTexture.localScale = 
							new Vector3(
								disabledTexture.localScale.x,
								(1.0f - (act.m_currentTimer/act.m_timerDuration)),
								disabledTexture.localScale.z);
					}
					else
					{
						disabledTexture.GetComponent<SpriteRenderer>().enabled = false;
					}
				}

			}

		}
	}

	void DisableObstaclesAbove()
	{
		m_world.DisableObstaclesAbove();
	}

	void DisableObstaclesBelow()
	{
		m_world.DisableObstaclesBelow();
	}

	void EnableObstacles()
	{
		m_world.EnableObstacles();
	}

	void OnGUI()
	{
		GUIUtils.DrawText("Catiitude Points: " + m_catiitudePoints + "/" + m_world.m_totalPoints,
		                  10,
		                  10,
		                  m_world.m_catiitudePointsStyle,
		                  false,
		                  true,
		                  1,
		                  Color.black);
	}
	
}
